package com.asgteach.bpcloud.views;

import com.gluonhq.charm.glisten.application.MobileApplication;
import com.gluonhq.charm.glisten.control.AppBar;
import com.gluonhq.charm.glisten.mvc.View;
import com.gluonhq.charm.glisten.visual.MaterialDesignIcon;
import com.asgteach.bpcloud.BPCloud;
import static com.asgteach.bpcloud.BPCloud.EDIT_VIEW;
import static com.asgteach.bpcloud.BPCloud.GRAPH_VIEW;
import static com.asgteach.bpcloud.BPCloud.STAT_VIEW;
import com.asgteach.bpcloud.model.BPData;
import com.asgteach.bpcloud.model.Model;
import com.asgteach.bpcloud.service.Service;
import com.gluonhq.charm.glisten.animation.BounceInLeftTransition;
import com.gluonhq.charm.glisten.control.Alert;
import com.gluonhq.charm.glisten.control.Avatar;
import com.gluonhq.charm.glisten.control.CharmListView;
import com.gluonhq.charm.glisten.control.FloatingActionButton;
import com.gluonhq.charm.glisten.control.LifecycleEvent;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.time.format.FormatStyle;
import java.util.Optional;
import javafx.beans.binding.When;
import javafx.collections.ListChangeListener;
import javafx.event.EventHandler;
import javafx.fxml.FXML;
import javafx.scene.control.Button;
import javafx.scene.control.ButtonType;
import javafx.scene.control.Label;
import javafx.scene.control.Tooltip;
import javafx.scene.image.Image;
import javax.inject.Inject;

public class ReadingsPresenter {

    private Button chartButton;
    private Button homeButton;
    private Button statButton;
    private static final String SIGN_IN_MESSAGE = "Please, sign in first\n"
            + "to gain online access\n"
            + "to BPCloud";

    private static final String NO_READINGS_MESSAGE = "There are no BP Readings.\n"
            + "Press '+' to create BP Readings.\n";

    @Inject
    private Model model;
    @Inject
    private Service service;

    @FXML
    private CharmListView<BPData, Double> bpCharmList;
    @FXML
    private View readings;

    public void initialize() {
        readings.setShowTransitionFactory(BounceInLeftTransition::new);
        readings.showingProperty().addListener((obs, oldValue, newValue) -> {
            if (newValue) {
                AppBar appBar = MobileApplication.getInstance().getAppBar();
                if (service.getUser() != null) {
                    appBar.setTitleText("BP: " + service.getUser().getName());                  
                } else {
                    appBar.setTitleText("BP Readings");
                }
                appBar.setNavIcon(new Avatar(17, new Image(BPCloud.class.getResourceAsStream("/icon.png"))));

                appBar.getActionItems().add(homeButton = MaterialDesignIcon.HOME.button(e
                        -> System.out.println("Home View")));
                appBar.getActionItems().add(chartButton = MaterialDesignIcon.TIMELINE.button(e
                        -> MobileApplication.getInstance().switchView(GRAPH_VIEW)));
                appBar.getActionItems().add(statButton = MaterialDesignIcon.VIEW_WEEK.button(e
                        -> MobileApplication.getInstance().switchView(STAT_VIEW)));

                homeButton.setTooltip(new Tooltip("VIEW DATA"));
                chartButton.setTooltip(new Tooltip("VIEW LINE CHART"));
                statButton.setTooltip(new Tooltip("VIEW STATS"));
                homeButton.setDisable(true);
            }
        });

        bpCharmList.setHeadersFunction(ReadingsPresenter::getDateNum);
        // sort headers most recent first
        bpCharmList.setHeaderComparator((d1, d2) -> d2.compareTo(d1));
        bpCharmList.setCellFactory(p -> new BPDataCell(this::edit, this::remove));
        bpCharmList.setHeaderCellFactory(p -> new BPHeaderCell());
        // sort list according to date/time--most recent first
        bpCharmList.setComparator((d1, d2) -> d2.getDate().compareTo(d1.getDate()));

        service.bpListProperty().addListener((ListChangeListener.Change<? extends BPData> c) -> {
            System.out.println("Service List Property listener: list size = " + c.getList().size());
            bpCharmList.setItems(service.bpListProperty().get());
//            if (service.bpListProperty().isEmpty()) {
//                MyData.fillBPData(service.bpListProperty().get());
//            }
        });

        final Label label = new Label(SIGN_IN_MESSAGE);
        label.textProperty().bind(new When(service.userProperty().isNotNull())
                .then(NO_READINGS_MESSAGE)
                .otherwise(SIGN_IN_MESSAGE));
        bpCharmList.setPlaceholder(label);

        FloatingActionButton fb = new FloatingActionButton();
        fb.textProperty().bind(new When(service.userProperty().isNotNull())
                .then(MaterialDesignIcon.ADD.text)
                .otherwise(MaterialDesignIcon.CLOUD_DOWNLOAD.text));

        fb.setOnAction(e -> {
            if (service.getUser() == null) {
                service.retrieveReadings();
            } else {
                // add new BP Data
                model.activeBPData().set(null);
                MobileApplication.getInstance().switchView(EDIT_VIEW);
            }
        });
        fb.showOn(readings);

        // service.retrieveReadings() will initiate user authentication.
        // The readings View must be shown beforehand; otherwise, it will
        // over-write the authentication View.
        // Once authenticated, you don't need to re-authenticate. 
        // Therefore, remove the Lifecycle event handler from the readings View.
        readings.addEventHandler(LifecycleEvent.SHOWN, new EventHandler<LifecycleEvent>() {
            @Override
            public void handle(LifecycleEvent event) {
                readings.removeEventHandler(LifecycleEvent.SHOWN, this);
                service.retrieveReadings();
            }
        });

    }

    private void edit(BPData data) {
        System.out.println("Edit for reading " + data);
        model.activeBPData().set(data);
        MobileApplication.getInstance().switchView(BPCloud.EDIT_VIEW);
    }

    private void remove(BPData data) {
        if (data == null) {
            System.out.println("Null data");
            return;
        }
        System.out.println("Remove confirmation for reading " + data);
        Alert alert = new Alert(javafx.scene.control.Alert.AlertType.CONFIRMATION);
        alert.setTitleText("Confirm Delete");
        alert.setContentText("OK to Delete BP Reading?\n\n"
                + data.getSystolic()
                + " / " + data.getDiastolic()
                + " mmHg\n" + data.getPulse()
                + " bpm\n" + data.getDate().format(DateTimeFormatter.ofLocalizedDateTime(FormatStyle.SHORT)));

        Button yes = new Button("Yes-Delete");
        yes.setOnAction(e -> {
            alert.setResult(ButtonType.YES);
            alert.hide();
        });
        yes.setDefaultButton(true);

        Button no = new Button("No");
        no.setCancelButton(true);
        no.setOnAction(e -> {
            alert.setResult(ButtonType.NO);
            alert.hide();
        });
        alert.getButtons().setAll(yes, no);

        Optional result = alert.showAndWait();
        if (result.isPresent() && result.get().equals(ButtonType.YES)) {
            System.out.println("removing item: " + data);
            service.removeReading(data);
        }

    }

    // Classify each BPData to a double based on year and month,
    // so all readings from the same year/month are grouped together under
    // the same heading.
    private static Double getDateNum(BPData data) {
        LocalDate d = data.getDate().toLocalDate();
        int month = d.getMonthValue();
        double number = d.getYear() + (month * .01);
        return number;
    }

}
