package com.asgteach.people.views;

import static com.asgteach.people.People.EDIT_VIEW;
import com.asgteach.people.model.Employee;
import com.asgteach.people.model.Model;
import com.asgteach.people.model.WorkForce;
import com.asgteach.people.service.RemoteService;
import com.gluonhq.charm.glisten.application.MobileApplication;
import com.gluonhq.charm.glisten.control.Alert;
import com.gluonhq.charm.glisten.control.AppBar;
import com.gluonhq.charm.glisten.control.CharmListCell;
import com.gluonhq.charm.glisten.control.CharmListView;
import com.gluonhq.charm.glisten.control.FloatingActionButton;
import com.gluonhq.charm.glisten.control.ListTile;
import com.gluonhq.charm.glisten.mvc.View;
import com.gluonhq.charm.glisten.visual.MaterialDesignIcon;
import com.gluonhq.connect.GluonObservableObject;
import java.util.Optional;
import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.collections.FXCollections;
import javafx.fxml.FXML;
import javafx.scene.control.Button;
import javafx.scene.control.ButtonType;
import javafx.scene.control.Label;
import javafx.scene.image.ImageView;
import javafx.scene.layout.HBox;
import javax.inject.Inject;

public class DisplayPresenter {

    @FXML
    private View display;

    @Inject
    private RemoteService remoteService;

    @Inject
    private Model model;

    @FXML
    private CharmListView<Employee, Integer> charmListView;

    private GluonObservableObject<WorkForce> workforce;
    private ChangeListener<? super Boolean> listener = null;

    public void initialize() {
        display.showingProperty().addListener((obs, oldValue, newValue) -> {
            if (newValue) {
                AppBar appBar = MobileApplication.getInstance().getAppBar();
                appBar.setNavIcon(MaterialDesignIcon.MENU.button(e
                        -> MobileApplication.getInstance().getDrawer().open()));
                appBar.setTitleText("Employee Display");
                appBar.getActionItems().add(
                        MaterialDesignIcon.REFRESH.button(e -> refreshList()));
                refreshList();

            }
        });
        charmListView.setPlaceholder(new Label("No items to show yet"));
        charmListView.setHeadersFunction(Employee::getDeptno);
        charmListView.setHeaderComparator((e1, e2) -> e1.compareTo(e2));
        charmListView.setComparator((Employee e1, Employee e2)
                -> String.CASE_INSENSITIVE_ORDER.compare(e1.getEname(), e2.getEname()));
        charmListView.setHeaderCellFactory(p -> new CharmListCell<Employee>() {

            private final Label label;

            {
                label = new Label();

            }

            @Override
            public void updateItem(Employee item, boolean empty) {
                super.updateItem(item, empty);
                if (item != null && !empty) {
                    label.setText("Dept:\t" + item.getDeptno());
                    setGraphic(label);
                } else {
                    setGraphic(null);
                }
            }
        });

        charmListView.setCellFactory(p -> new CharmListCell<Employee>() {

            private final ListTile tile;
            private final ImageView imageView;
            private final Button buttonEdit;
            private final Button buttonDel;
            private Employee currentItem;
                    
            {
                tile = new ListTile();
                imageView = new ImageView();
                tile.setPrimaryGraphic(imageView);
                buttonEdit = MaterialDesignIcon.EDIT.button(e -> edit(currentItem));
                buttonDel = MaterialDesignIcon.DELETE.button(e -> remove(currentItem));
                tile.setSecondaryGraphic(new HBox(buttonEdit, buttonDel));
            }

            @Override
            public void updateItem(Employee item, boolean empty) {
                super.updateItem(item, empty);
                if (item != null && !empty) {
                    currentItem = item;
                    tile.textProperty().setAll(item.getEname(),
                            item.getJob(),
                            "Hired: " + item.getHiredate().substring(0, 10)
                            + "\nSalary: " + item.getSal());
                    setGraphic(tile);
                } else {
                    setGraphic(null);
                }
            }

        });
        charmListView.setItems(FXCollections.emptyObservableList());
        refreshList();
        FloatingActionButton fb = new FloatingActionButton();
        fb.textProperty().set(MaterialDesignIcon.ADD.text);
        fb.setOnAction(e -> {
            // add new Employee
            model.editMode.set(false);
            MobileApplication.getInstance().switchView(EDIT_VIEW);
        });
        fb.showOn(display);

    }

    private void refreshList() {       
        workforce = remoteService.getEmployees(WorkForce.class);
        if (listener != null) {
            workforce.initializedProperty().removeListener(listener);
        }
        workforce.initializedProperty().addListener(listener = (ObservableValue<? extends Boolean> obsrv, Boolean ov, Boolean nv) -> {
            if (nv) {
                charmListView.setItems(FXCollections.observableArrayList(workforce.get().getItems()));        
            }
        });
    }

    private void edit(Employee data) {
        model.activeEmployee.set(data);
        model.editMode.set(true);
        System.out.println("Edit for reading " + model.activeEmployee.get());
        MobileApplication.getInstance().switchView(EDIT_VIEW);
    }

    private void remove(Employee data) {
        if (data == null) {
            System.out.println("Null data");
            return;
        }
        Alert alert = new Alert(javafx.scene.control.Alert.AlertType.CONFIRMATION);
        alert.setTitleText("Confirm Delete");
        alert.setContentText("OK to Delete Employee Record\n\n"
                + data.getEname()
                + " / " + data.getEmpno());

        Button yes = new Button("Yes-Delete");
        yes.setOnAction(e -> {
            alert.setResult(ButtonType.YES);
            alert.hide();
        });
        yes.setDefaultButton(true);

        Button no = new Button("No");
        no.setCancelButton(true);
        no.setOnAction(e -> {
            alert.setResult(ButtonType.NO);
            alert.hide();
        });
        alert.getButtons().setAll(yes, no);

        Optional result = alert.showAndWait();
        if (result.isPresent() && result.get().equals(ButtonType.YES)) {
            System.out.println("removing item: " + data);
            GluonObservableObject<String> dataResult = remoteService.deleteEmployee(String.class, data.getEmpno() + "");
            dataResult.initializedProperty().addListener((obs, ov, nv) -> {
                if (nv) {
                    if (dataResult.get().contains("ORA")) {
                        Alert alertErr = new Alert(javafx.scene.control.Alert.AlertType.ERROR);
                        alertErr.setTitleText("Oracle Database Error");
                        alertErr.setContentText(String.format("%s\n%s",
                                "Error:",
                                dataResult.get()));
                        alertErr.showAndWait();
                    } else {
                        Alert okAlert = new Alert(javafx.scene.control.Alert.AlertType.INFORMATION);
                        okAlert.setTitleText("Employee Removed");
                        okAlert.setContentText(String.format("%s",
                                data));
                        refreshList();
                        okAlert.showAndWait();
                    }
                }
            });
        }
    }
}
